(*  Title:      Keyserver.thy
    SPDX-License-Identifier: BSD-3-Clause
*)

theory Keyserver_4_1
  imports 
"Automated_Stateful_Protocol_Verification.PSPSP"
begin

declare [[pspsp_timing]]

trac\<open>
Protocol: keyserver

Enumerations:
honest = {a,b,c,d}
dishonest = {i}
agent = honest ++ dishonest

Sets:
ring/1 valid/1 revoked/1 deleted/1

Functions:
Public sign/2 crypt/2 pair/2
Private inv/1

Analysis:
sign(X,Y) -> Y
crypt(X,Y) ? inv(X) -> Y
pair(X,Y) -> X,Y

Transactions:
# Out-of-band registration
outOfBand(A:honest)
  new PK
  insert PK ring(A)
  insert PK valid(A)
  send PK.

# Out-of-band registration (for dishonest users; they reveal their private keys to the intruder)
oufOfBandD(A:dishonest)
  new PK
  insert PK valid(A)
  send PK
  send inv(PK).

# User update key
keyUpdateUser(A:honest,PK:value)
  PK in ring(A)
  new NPK
  delete PK ring(A)
  insert PK deleted(A)
  insert NPK ring(A)
  send sign(inv(PK),pair(A,NPK)).

# Server update key
keyUpdateServer(A:agent,PK:value,NPK:value)
  receive sign(inv(PK),pair(A,NPK))
  PK in valid(A)
  NPK notin valid(_)
  NPK notin revoked(_)
  delete PK valid(A)
  insert PK revoked(A)
  insert NPK valid(A)
  send inv(PK).

# Attack definition
attackDef(A:honest,PK:value)
  receive inv(PK)
  PK in valid(A)
  attack.
\<close>


section \<open>Proof of security\<close>
protocol_model_setup spm: keyserver
compute_fixpoint keyserver_protocol keyserver_fixpoint
value "let (FP,_,TI) = keyserver_fixpoint in (size FP, size TI)"
protocol_security_proof [unsafe] ssp: keyserver


section \<open>The generated theorems and definitions\<close>
thm ssp.protocol_secure
thm spm.constraint_model_def
thm spm.reachable_constraints.simps

thm keyserver_enum_consts.nchotomy
thm keyserver_sets.nchotomy
thm keyserver_fun.nchotomy
thm keyserver_atom.nchotomy
thm keyserver_arity.simps
thm keyserver_public.simps
thm keyserver_\<Gamma>.simps
thm keyserver_Ana.simps

thm keyserver_protocol_def
thm keyserver_transaction_outOfBand_def
thm keyserver_transaction_keyUpdateUser_def
thm keyserver_transaction_keyUpdateServer_def
thm keyserver_transaction_attackDef_def

thm keyserver_fixpoint_def

end
