(*  Title:      Keyserver_Composition.thy
    SPDX-License-Identifier: BSD-3-Clause
*)

theory Keyserver_Composition_2_1
  imports 
"Automated_Stateful_Protocol_Verification.PSPSP"
begin

declare [[pspsp_timing]]

trac\<open>
Protocol: kscomp

Enumerations:
honest = {a,b}
dishonest = {i}
agent = honest ++ dishonest

Sets:
ring/1 valid/1 revoked/1 deleted/1
ring'/1 seen/1 pubkeys/0

Functions:
Public h/1 sign/2 crypt/2 scrypt/2 pair/2 update/3
Private inv/1 pw/1

Analysis:
sign(X,Y) -> Y
crypt(X,Y) ? inv(X) -> Y
scrypt(X,Y) ? X -> Y
pair(X,Y) -> X,Y
update(X,Y,Z) -> X,Y,Z

Transactions:
### The signature-based keyserver protocol
p1_outOfBand(A:honest)
new PK
insert PK ring(A)
insert PK valid(A)
send PK.

p1_oufOfBandD(A:dishonest)
new PK
insert PK valid(A)
send PK
send inv(PK).

p1_keyUpdateUser(A:honest,PK:value)
PK in ring(A)
new NPK
delete PK ring(A)
insert PK deleted(A)
insert NPK ring(A)
send sign(inv(PK),pair(A,NPK)).

p1_keyUpdateServer(A:agent,PK:value,NPK:value)
receive sign(inv(PK),pair(A,NPK))
PK in valid(A)
NPK notin valid(_)
NPK notin revoked(_)
delete PK valid(A)
insert PK revoked(A)
insert NPK valid(A)
send inv(PK).

p1_attackDef(A:honest,PK:value)
receive inv(PK)
PK in valid(A)
attack.

### The password-based keyserver protocol
p2_passwordGenD(A:dishonest)
send pw(A).

p2_pubkeysGen()
new PK
insert PK pubkeys
send PK.

p2_updateKeyPw(A:honest,PK:value)
PK in pubkeys
new NPK
insert NPK ring'(A)
send NPK
send crypt(PK,update(A,NPK,pw(A))).

p2_updateKeyServerPw(A:agent,PK:value,NPK:value)
receive crypt(PK,update(A,NPK,pw(A)))
PK in pubkeys
NPK notin pubkeys
NPK notin seen(_)
insert NPK valid(A)
insert NPK seen(A).

p2_authAttack2(A:honest,PK:value)
receive inv(PK)
PK in valid(A)
attack.
\<close>


section \<open>Proof: The composition of the two keyserver protocols is secure\<close>
protocol_model_setup spm: kscomp
compute_fixpoint kscomp_protocol kscomp_fixpoint
value "let (FP,_,TI) = kscomp_fixpoint in (size FP, size TI)"
protocol_security_proof [nbe] ssp: kscomp


section \<open>The generated theorems and definitions\<close>
thm ssp.protocol_secure

thm kscomp_enum_consts.nchotomy
thm kscomp_sets.nchotomy
thm kscomp_fun.nchotomy
thm kscomp_atom.nchotomy
thm kscomp_arity.simps
thm kscomp_public.simps
thm kscomp_\<Gamma>.simps
thm kscomp_Ana.simps

thm kscomp_transaction_p1_outOfBand_def
thm kscomp_transaction_p1_oufOfBandD_def
thm kscomp_transaction_p1_keyUpdateUser_def
thm kscomp_transaction_p1_keyUpdateServer_def
thm kscomp_transaction_p1_attackDef_def
thm kscomp_transaction_p2_passwordGenD_def
thm kscomp_transaction_p2_pubkeysGen_def
thm kscomp_transaction_p2_updateKeyPw_def
thm kscomp_transaction_p2_updateKeyServerPw_def
thm kscomp_transaction_p2_authAttack2_def
thm kscomp_protocol_def

thm kscomp_fixpoint_def

end
